@@ -64,6 +64,8 @@ class Backend(GridObjects, ABC):
         Thermal limit of the powerline in amps for each powerline. Thie thermal limit is relevant on only one
         side of the powerline: the same side returned by :func:`Backend.get_line_overflow`
     """
+    env_name = "unknown"
+
     def __init__(self, detailed_infos_for_cascading_failures=False):
         """
         Initialize an instance of Backend. This does nothing per se. Only the call to :func:`Backend.load_grid`
@@ -94,6 +96,7 @@ class Backend(GridObjects, ABC):
         :raise: :class:`grid2op.Exceptions.EnvError` and possibly all of its derived class.
         """
         # test the results gives the proper size
+        self.__class__ = self.init_grid(self)
         tmp = self.get_line_status()
         if tmp.shape[0] != self.n_line:
             raise IncorrectNumberOfLines("returned by \"backend.get_line_status()\"")
@@ -610,6 +613,16 @@ class Backend(GridObjects, ABC):
 
         """
         infos = []
+        # print("\t\t gen p_mw {}".format(np.sum(self._grid.gen["p_mw"])))  # OK
+        # print("\t\t gen vm_pu {}".format(np.sum(self._grid.gen["vm_pu"])))  # OK
+        # print("\t\t load p_mw {}".format(np.sum(self._grid.load["p_mw"])))  # OK
+        # print("\t\t line in_service {}".format(np.sum(self._grid.line["in_service"])))  # OK
+        # print("\t\t shunt in_service {}".format(np.sum(self._grid.shunt["in_service"])))  # OK
+        # print("\t\t shunt q {}".format(np.sum(self._grid.shunt["q_mvar"])))  # OK
+        # print("\t\t load q_mvar {}".format(np.sum(self._grid.load["q_mvar"])))  # OK
+        # print("slack bus {}".format(self._grid["ext_grid"]))  # OK
+        # print("slack p {}".format(self._grid._ppc["gen"][self._iref_slack, 1]))
+        # self._nb_bus_before = None
         self._runpf_with_diverging_exception(is_dc)
 
         disconnected_during_cf = np.full(self.n_line, fill_value=False, dtype=dt_bool)
@@ -672,7 +685,6 @@ class Backend(GridObjects, ABC):
         p_ex, q_ex, v_ex, *_ = self.lines_ex_info()
         p_gen, q_gen, v_gen = self.generators_info()
         p_load, q_load, v_load = self.loads_info()
-        p_s, q_s, v_s, bus_s = self.shunt_info()
 
         # fist check the "substation law" : nothing is created at any substation
         p_subs = np.zeros(self.n_sub)
@@ -717,14 +729,15 @@ class Backend(GridObjects, ABC):
             q_bus[self.load_to_subid[i],  topo_vect[self.load_pos_topo_vect[i]]-1] += q_load[i]
 
         if self.shunts_data_available:
-            for i in range(len(p_s)):
-                tmp_bus = bus_s[i]
-                sub_id = self.sub_from_bus_id(tmp_bus)
-                p_subs[sub_id] += p_s[i]
-                q_subs[sub_id] += q_s[i]
+            p_s, q_s, v_s, bus_s = self.shunt_info()
+            for i in range(self.n_shunt):
+                # for substations
+                p_subs[self.shunt_to_subid[i]] += p_s[i]
+                q_subs[self.shunt_to_subid[i]] += q_s[i]
 
-                p_bus[sub_id, 1*(tmp_bus!=sub_id)] += p_s[i]
-                q_bus[sub_id, 1*(tmp_bus!=sub_id)] += q_s[i]
+                # for buses
+                p_bus[self.shunt_to_subid[i], bus_s[i] - 1] += p_s[i]
+                q_bus[self.shunt_to_subid[i], bus_s[i] - 1] += q_s[i]
         else:
             warnings.warn("Backend.check_kirchoff Impossible to get shunt information. Reactive information might be "
                           "incorrect.")
@@ -838,7 +851,8 @@ class Backend(GridObjects, ABC):
         topo_vect = self.get_topo_vect()
         prod_p, _, prod_v = self.generators_info()
         load_p, load_q, _ = self.loads_info()
-        set_me = CompleteAction(self)
+        complete_action_class = CompleteAction.init_grid(self)
+        set_me = complete_action_class(self)
         set_me.update({"set_line_status": line_status,
                        "set_bus": topo_vect})
 
