@@ -10,9 +10,11 @@ import os
 import copy
 import numpy as np
 import pandas as pd
+import warnings
 from datetime import datetime, timedelta
 import pdb
 
+from grid2op.dtypes import dt_int, dt_float, dt_bool
 from grid2op.Exceptions import IncorrectNumberOfElements, ChronicsError, ChronicsNotFoundError
 from grid2op.Exceptions import IncorrectNumberOfLoads, IncorrectNumberOfGenerators, IncorrectNumberOfLines
 from grid2op.Exceptions import EnvError, InsufficientData
@@ -132,6 +134,11 @@ class GridStateFromFile(GridValue):
         self._order_hazards = None
         self._order_maintenance = None
 
+        # order of the names in the backend
+        self._order_backend_loads = None
+        self._order_backend_prods = None
+        self._order_backend_lines = None
+
     def _assert_correct(self, dict_convert, order_backend):
         len_backend = len(order_backend)
         len_dict_keys = len(dict_convert)
@@ -170,6 +177,7 @@ class GridStateFromFile(GridValue):
                                             "it's composed of only one line with a datetime in the \"%Y-%m-%d %H:%M\""
                                             "format.")
             self.start_datetime = tmp
+            self.current_datetime = tmp
 
         if os.path.exists(os.path.join(self.path, "time_interval.info")):
             with open(os.path.join(self.path, "time_interval.info"), "r") as f:
@@ -206,7 +214,7 @@ class GridStateFromFile(GridValue):
         file_ext = self._get_fileext(data_name)
         if file_ext is not None:
             res = pd.read_csv(os.path.join(self.path, "{}{}".format(data_name, file_ext)),
-                        sep=self.sep, chunksize=self.chunk_size)
+                              sep=self.sep, chunksize=self.chunk_size)
         else:
             res = None
         return res
@@ -224,31 +232,31 @@ class GridStateFromFile(GridValue):
         if load_p is not None:
             self._assert_correct_second_stage(load_p.columns, self.names_chronics_to_backend, "loads", "active")
             order_chronics_load_p = np.array([order_backend_loads[self.names_chronics_to_backend["loads"][el]]
-                                              for el in load_p.columns]).astype(np.int)
+                                              for el in load_p.columns]).astype(dt_int)
         if load_q is not None:
             self._assert_correct_second_stage(load_q.columns, self.names_chronics_to_backend, "loads", "reactive")
             order_backend_load_q = np.array([order_backend_loads[self.names_chronics_to_backend["loads"][el]]
-                                             for el in load_q.columns]).astype(np.int)
+                                             for el in load_q.columns]).astype(dt_int)
 
         if prod_p is not None:
             self._assert_correct_second_stage(prod_p.columns, self.names_chronics_to_backend, "prods", "active")
             order_backend_prod_p = np.array([order_backend_prods[self.names_chronics_to_backend["prods"][el]]
-                                             for el in prod_p.columns]).astype(np.int)
+                                             for el in prod_p.columns]).astype(dt_int)
 
         if prod_v is not None:
             self._assert_correct_second_stage(prod_v.columns, self.names_chronics_to_backend, "prods", "voltage magnitude")
             order_backend_prod_v = np.array([order_backend_prods[self.names_chronics_to_backend["prods"][el]]
-                                             for el in prod_v.columns]).astype(np.int)
+                                             for el in prod_v.columns]).astype(dt_int)
 
         if hazards is not None:
             self._assert_correct_second_stage(hazards.columns, self.names_chronics_to_backend, "lines", "hazards")
             order_backend_hazards = np.array([order_backend_lines[self.names_chronics_to_backend["lines"][el]]
-                                              for el in hazards.columns]).astype(np.int)
+                                              for el in hazards.columns]).astype(dt_int)
 
         if maintenance is not None:
             self._assert_correct_second_stage(maintenance.columns, self.names_chronics_to_backend, "lines", "maintenance")
             order_backend_maintenance = np.array([order_backend_lines[self.names_chronics_to_backend["lines"][el]]
-                                                  for el in maintenance.columns]).astype(np.int)
+                                                  for el in maintenance.columns]).astype(dt_int)
 
         return order_chronics_load_p, order_backend_load_q, \
                order_backend_prod_p, order_backend_prod_v, \
@@ -320,6 +328,10 @@ class GridStateFromFile(GridValue):
         self.n_load = len(order_backend_loads)
         self.n_line = len(order_backend_lines)
 
+        self._order_backend_loads = order_backend_loads
+        self._order_backend_prods = order_backend_prods
+        self._order_backend_lines = order_backend_lines
+
         self.names_chronics_to_backend = copy.deepcopy(names_chronics_to_backend)
         if self.names_chronics_to_backend is None:
             self.names_chronics_to_backend = {}
@@ -350,14 +362,14 @@ class GridStateFromFile(GridValue):
         read_compressed = self._get_fileext("hazards")
         if read_compressed is not None:
             hazards = pd.read_csv(os.path.join(self.path, "hazards{}".format(read_compressed)),
-                                       sep=self.sep)
+                                  sep=self.sep)
         else:
             hazards = None
 
         read_compressed = self._get_fileext("maintenance")
         if read_compressed is not None:
             maintenance = pd.read_csv(os.path.join(self.path, "maintenance{}".format(read_compressed)),
-                                           sep=self.sep)
+                                      sep=self.sep)
         else:
             maintenance = None
 
@@ -457,20 +469,20 @@ class GridStateFromFile(GridValue):
         self.maintenance_duration = None
 
         if load_p is not None:
-            self.load_p = copy.deepcopy(load_p.values[:, self._order_load_p])
+            self.load_p = copy.deepcopy(load_p.values[:, self._order_load_p].astype(dt_float))
         if load_q is not None:
-            self.load_q = copy.deepcopy(load_q.values[:, self._order_load_q])
+            self.load_q = copy.deepcopy(load_q.values[:, self._order_load_q].astype(dt_float))
         if prod_p is not None:
-            self.prod_p = copy.deepcopy(prod_p.values[:, self._order_prod_p])
+            self.prod_p = copy.deepcopy(prod_p.values[:, self._order_prod_p].astype(dt_float))
         if prod_v is not None:
-            self.prod_v = copy.deepcopy(prod_v.values[:, self._order_prod_v])
+            self.prod_v = copy.deepcopy(prod_v.values[:, self._order_prod_v].astype(dt_float))
 
         # TODO optimize this piece of code, and the whole laoding process if hazards.csv and maintenance.csv are
         # provided in the proper format.
         if hazards is not None:
             # hazards and maintenance cannot be computed by chunk. So we need to differenciate their behaviour
             self.hazards = copy.deepcopy(hazards.values[:, self._order_hazards])
-            self.hazard_duration = np.zeros(shape=(self.hazards.shape[0], self.n_line), dtype=np.int)
+            self.hazard_duration = np.zeros(shape=(self.hazards.shape[0], self.n_line), dtype=dt_int)
             for line_id in range(self.n_line):
                 self.hazard_duration[:, line_id] = self.get_hazard_duration_1d(self.hazards[:, line_id])
 
@@ -478,8 +490,8 @@ class GridStateFromFile(GridValue):
 
         if maintenance is not None:
             self.maintenance = copy.deepcopy(maintenance.values[:, self._order_maintenance])
-            self.maintenance_time = np.zeros(shape=(self.maintenance.shape[0], self.n_line), dtype=np.int) - 1
-            self.maintenance_duration = np.zeros(shape=(self.maintenance.shape[0], self.n_line), dtype=np.int)
+            self.maintenance_time = np.zeros(shape=(self.maintenance.shape[0], self.n_line), dtype=dt_int) - 1
+            self.maintenance_duration = np.zeros(shape=(self.maintenance.shape[0], self.n_line), dtype=dt_int)
 
             # test that with chunk size
             for line_id in range(self.n_line):
@@ -488,6 +500,7 @@ class GridStateFromFile(GridValue):
 
             # there are _maintenance and hazards only if the value in the file is not 0.
             self.maintenance = self.maintenance != 0.
+            self.maintenance = self.maintenance.astype(dt_bool)
 
     def done(self):
         """
@@ -553,7 +566,7 @@ class GridStateFromFile(GridValue):
         dict_ = {}
         prod_v = None
         if self.load_p is not None:
-            dict_["load_p"] =  1.0 * self.load_p[self.current_index, :]
+            dict_["load_p"] = 1.0 * self.load_p[self.current_index, :]
         if self.load_q is not None:
             dict_["load_q"] = 1.0 * self.load_q[self.current_index, :]
         if self.prod_p is not None:
@@ -573,16 +586,16 @@ class GridStateFromFile(GridValue):
         self.curr_iter += 1
 
         if self.maintenance_time is not None:
-            maintenance_time = 1 * self.maintenance_time[self.current_index, :]
-            maintenance_duration = 1 * self.maintenance_duration[self.current_index, :]
+            maintenance_time = dt_int(1 * self.maintenance_time[self.current_index, :])
+            maintenance_duration = dt_int(1 * self.maintenance_duration[self.current_index, :])
         else:
-            maintenance_time = np.full(self.n_line, fill_value=-1, dtype=np.int)
-            maintenance_duration = np.full(self.n_line, fill_value=0, dtype=np.int)
+            maintenance_time = np.full(self.n_line, fill_value=-1, dtype=dt_int)
+            maintenance_duration = np.full(self.n_line, fill_value=0, dtype=dt_int)
 
         if self.hazard_duration is not None:
             hazard_duration = 1 * self.hazard_duration[self.current_index, :]
         else:
-            hazard_duration = np.full(self.n_line, fill_value=-1, dtype=np.int)
+            hazard_duration = np.full(self.n_line, fill_value=-1, dtype=dt_int)
 
         return self.current_datetime, res, maintenance_time, maintenance_duration, hazard_duration, prod_v
 
@@ -685,3 +698,145 @@ class GridStateFromFile(GridValue):
 
     def set_chunk_size(self, new_chunk_size):
         self.chunk_size = new_chunk_size
+
+    def _convert_datetime(self, datetime_beg):
+        res = datetime_beg
+        if not isinstance(datetime_beg, datetime):
+            try:
+                res = datetime.strptime(datetime_beg, "%Y-%m-%d %H:%M")
+            except:
+                try:
+                    res = datetime.strptime(datetime_beg, "%Y-%m-%d")
+                except:
+                    raise ChronicsError("Impossible to convert \"{}\" to a valid datetime. Accepted format is "
+                                        "\"%Y-%m-%d %H:%M\"".format(datetime_beg))
+        return res
+
+    def _extract_array(self, nm):
+        var = self.__dict__[nm]
+        if var is None:
+            return None
+        else:
+            return var[self.current_index,:]
+
+    def _save_array(self, array_, path_out, name, colnames):
+        if array_ is None:
+            return
+        tmp = pd.DataFrame(array_)
+        tmp.columns = colnames
+        tmp.to_csv(os.path.join(path_out, name), index=False, sep=self.sep)
+
+    def _init_res_split(self, nb_rows):
+        res_prod_p = None
+        res_prod_v = None
+        res_load_p = None
+        res_load_q = None
+        res_maintenance = None
+        res_hazards = None
+        if self.prod_p is not None:
+            res_prod_p = np.zeros((nb_rows, self.n_gen), dtype=dt_float)
+        if self.prod_v is not None:
+            res_prod_v = np.zeros((nb_rows, self.n_gen), dtype=dt_float)
+        if self.load_p is not None:
+            res_load_p = np.zeros((nb_rows, self.n_load), dtype=dt_float)
+        if self.load_q is not None:
+            res_load_q = np.zeros((nb_rows, self.n_load), dtype=dt_float)
+        if self.maintenance is not None:
+            res_maintenance = np.zeros((nb_rows, self.n_line), dtype=dt_float)
+        if self.hazards is not None:
+            res_hazards = np.zeros((nb_rows, self.n_line), dtype=dt_float)
+        return res_prod_p, res_prod_v, res_load_p, res_load_q, res_maintenance, res_hazards
+
+    def _update_res_split(self, i, tmp, *arrays):
+        res_prod_p, res_prod_v, res_load_p, res_load_q, res_maintenance, res_hazards = arrays
+        if res_prod_p is not None:
+            res_prod_p[i, :] = tmp._extract_array("prod_p")
+        if res_prod_v is not None:
+            res_prod_v[i, :] = tmp._extract_array("prod_v")
+        if res_load_p is not None:
+            res_load_p[i, :] = tmp._extract_array("load_p")
+        if res_load_q is not None:
+            res_load_q[i, :] = tmp._extract_array("load_q")
+        if res_maintenance is not None:
+            res_maintenance[i, :] = tmp._extract_array("maintenance")
+        if res_hazards is not None:
+            res_hazards[i, :] = tmp._extract_array("hazards")
+
+    def _clean_arrays(self, i, *arrays):
+        res_prod_p, res_prod_v, res_load_p, res_load_q, res_maintenance, res_hazards = arrays
+        if res_prod_p is not None:
+            res_prod_p = res_prod_p[:i, :]
+        if res_prod_v is not None:
+            res_prod_v = res_prod_v[:i, :]
+        if res_load_p is not None:
+            res_load_p = res_load_p[:i, :]
+        if res_load_q is not None:
+            res_load_q = res_load_q[:i, :]
+        if res_maintenance is not None:
+            res_maintenance = res_maintenance[:i, :]
+        if res_hazards is not None:
+            res_hazards = res_hazards[:i, :]
+        return res_prod_p, res_prod_v, res_load_p, res_load_q, res_maintenance, res_hazards
+
+    def _get_name_arrays_for_saving(self):
+        return ["prod_p", "prod_v", "load_p", "load_q", "maintenance", "hazards"]
+
+    def _get_colorder_arrays_for_saving(self):
+        return [self._order_backend_prods, self._order_backend_prods,
+                self._order_backend_loads, self._order_backend_loads,
+                self._order_backend_lines, self._order_backend_lines]
+
+    def split_and_save(self, datetime_beg, datetime_end, path_out):
+        # work on a copy of myself
+        tmp = copy.deepcopy(self)
+        datetime_beg = self._convert_datetime(datetime_beg)
+        datetime_end = self._convert_datetime(datetime_end)
+
+        nb_rows = datetime_end - datetime_beg
+        nb_rows = nb_rows.total_seconds()
+        nb_rows = int(nb_rows / self.time_interval.total_seconds())+1
+        if nb_rows <= 0:
+            raise ChronicsError("Invalid time step to be extracted. Make sure \"datetime_beg\" is lower than "
+                                "\"datetime_end\"")
+
+        # prepare folder
+        if not os.path.exists(path_out):
+            os.mkdir(path_out)
+
+        # skip until datetime_beg starts
+        curr_dt = tmp.current_datetime
+        if curr_dt > datetime_beg:
+            warnings.warn("split_and_save: you ask for a beginning of the extraction of the chronics after the "
+                          "current datetime of it. If they ever existed, the data in the chronics prior to {}"
+                          "will be ignored".format(curr_dt))
+        # in the chronics we load the first row to initialize the data, so here we stop just a bit before that
+        datetime_start = datetime_beg - self.time_interval
+        while curr_dt < datetime_start:
+            curr_dt, *_ = tmp.load_next()
+        real_init_dt = curr_dt
+
+        arrays = self._init_res_split(nb_rows)
+        i = 0
+        while curr_dt < datetime_end:
+            self._update_res_split(i, tmp, *arrays)
+            curr_dt, *_ = tmp.load_next()
+            i += 1
+        if i < nb_rows:
+            warnings.warn("split_and_save: chronics goes up to {} but you want to split it up to {}. Results "
+                          "has been troncated".format(curr_dt, datetime_end))
+
+        arrays = self._clean_arrays(i, *arrays)
+        nms = self._get_name_arrays_for_saving()
+        orders_columns = self._get_colorder_arrays_for_saving()
+        for el, nm, colnames in zip(arrays,
+                                    nms,
+                                    orders_columns):
+            nm = "{}{}".format(nm, ".csv.bz2")
+            self._save_array(el, path_out, nm, colnames)
+
+        with open(os.path.join(path_out, "start_datetime.info"), "w") as f:
+            f.write("{:%Y-%m-%d %H:%M}\n".format(real_init_dt))
+
+        tmp_for_time_delta = datetime(year=2018, month=1, day=1, hour=0, minute=0, second=0) + self.time_interval
+        with open(os.path.join(path_out, "time_interval.info"), "w") as f:
+            f.write("{:%H:%M}\n".format(tmp_for_time_delta))
