@@ -8,8 +8,8 @@
 import numpy as np
 import os
 import copy
+import warnings
 import pdb
-import collections
 
 from grid2op.dtypes import dt_int, dt_float, dt_bool
 from grid2op.Action import ActionSpace, BaseAction, TopologyAction, DontAct, CompleteAction
@@ -114,6 +114,7 @@ class Environment(BaseEnv):
                  chronics_handler,
                  backend,
                  parameters,
+                 name="unknown",
                  names_chronics_to_backend=None,
                  actionClass=TopologyAction,
                  observationClass=CompleteObservation,
@@ -134,6 +135,10 @@ class Environment(BaseEnv):
                            epsilon_poly=epsilon_poly,
                            tol_poly=tol_poly,
                            other_rewards=other_rewards)
+        if name == "unknown":
+            warnings.warn("It is NOT recommended to create an environment without \"make\" and EVEN LESS "
+                          "to use an environment without a name")
+        self.name = name
 
         # the voltage controler
         self.voltagecontrolerClass = voltagecontrolerClass
@@ -185,10 +190,8 @@ class Environment(BaseEnv):
                                    "and not an object (an instance of a class). "
                                    "It is currently \"{}\"".format(type(rewardClass)))
         if not issubclass(rewardClass, BaseReward):
-            raise Grid2OpException(
-                "Parameter \"rewardClass\" used to build the Environment should derived form the grid2op.BaseReward class, "
-                "type provided is \"{}\"".format(
-                    type(rewardClass)))
+            raise Grid2OpException("Parameter \"rewardClass\" used to build the Environment should derived form "
+                                   "the grid2op.BaseReward class, type provided is \"{}\"".format(type(rewardClass)))
         self.rewardClass = rewardClass
         self.actionClass = actionClass
         self.observationClass = observationClass
@@ -197,18 +200,17 @@ class Environment(BaseEnv):
         self.init_grid_path = os.path.abspath(init_grid_path)
 
         if not isinstance(backend, Backend):
-            raise Grid2OpException(
-                "Parameter \"backend\" used to build the Environment should derived form the grid2op.Backend class, "
-                "type provided is \"{}\"".format(
-                    type(backend)))
+            raise Grid2OpException( "Parameter \"backend\" used to build the Environment should derived form the "
+                                    "grid2op.Backend class, type provided is \"{}\"".format(type(backend)))
         self.backend = backend
         self.backend.load_grid(self.init_grid_path)  # the real powergrid of the environment
 
         self.backend.load_redispacthing_data(os.path.split(self.init_grid_path)[0])
         self.backend.load_grid_layout(os.path.split(self.init_grid_path)[0])
+        self.backend.set_env_name(self.name)
 
         self.backend.assert_grid_correct()
-        self.init_grid(backend)
+
         self._has_been_initialized()  # really important to include this piece of code!
 
         if self._thermal_limit_a is None:
@@ -253,19 +255,20 @@ class Environment(BaseEnv):
                     type(observationClass)))
 
         # action affecting the grid that will be made by the agent
-        self.helper_action_player = ActionSpace(gridobj=self.backend,
-                                                actionClass=actionClass,
-                                                legal_action=self.game_rules.legal_action)
+        self.helper_action_class = ActionSpace.init_grid(gridobj=self.backend)
+        self.helper_action_player = self.helper_action_class(gridobj=self.backend,
+                                                             actionClass=actionClass,
+                                                             legal_action=self.game_rules.legal_action)
 
         # action that affect the grid made by the environment.
-        self.helper_action_env = ActionSpace(gridobj=self.backend,
-                                             actionClass=CompleteAction,
-                                             legal_action=self.game_rules.legal_action)
-
-        self.helper_observation = ObservationSpace(gridobj=self.backend,
-                                                   observationClass=observationClass,
-                                                   rewardClass=rewardClass,
-                                                   env=self)
+        self.helper_action_env = self.helper_action_class(gridobj=self.backend,
+                                                          actionClass=CompleteAction,
+                                                          legal_action=self.game_rules.legal_action)
+        self.helper_observation_class = ObservationSpace.init_grid(gridobj=self.backend)
+        self.helper_observation = self.helper_observation_class(gridobj=self.backend,
+                                                                observationClass=observationClass,
+                                                                rewardClass=rewardClass,
+                                                                env=self)
 
         # handles input data
         if not isinstance(chronics_handler, ChronicsHandler):
@@ -346,10 +349,11 @@ class Environment(BaseEnv):
             The voltages that has been specified in the chronics
 
         """
-        self.env_modification += self.voltage_controler.fix_voltage(self.current_obs,
-                                                                    agent_action,
-                                                                    self.env_modification,
-                                                                    prod_v_chronics)
+        volt_control_act = self.voltage_controler.fix_voltage(self.current_obs,
+                                                              agent_action,
+                                                              self.env_modification,
+                                                              prod_v_chronics)
+        return volt_control_act
 
     def set_chunk_size(self, new_chunk_size):
         """
@@ -452,12 +456,6 @@ class Environment(BaseEnv):
                       "Please install matplotlib or run pip install grid2op[optional]"
             raise Grid2OpException(err_msg) from None
 
-        if graph_layout is None:
-            # Grid layout must be ordered by keys in order to display correctly
-            # Apparently sub stations indexes to grid_layout key is in alphabetical order
-            graph_layout = collections.OrderedDict(sorted(self.helper_observation.grid_layout.items()))
-
-        self.helper_observation.grid_layout = graph_layout
         self.viewer = PlotMatplot(self.helper_observation)
         self.viewer_fig = None
         # Set renderer modes
@@ -484,6 +482,7 @@ class Environment(BaseEnv):
         if self._thermal_limit_a is not None:
             self.backend.set_thermal_limit(self._thermal_limit_a.astype(dt_float))
 
+        self._backend_action = self._backend_action_class()
         do_nothing = self.helper_action_env({})
         *_, fail_to_start, info = self.step(do_nothing)
         if fail_to_start:
@@ -491,7 +490,7 @@ class Environment(BaseEnv):
                                    "Available information are: {}".format(info))
 
         # test the backend returns object of the proper size
-        self.backend.assert_grid_correct_after_powerflow()
+        # self.backend.assert_grid_correct_after_powerflow()
 
     def add_text_logger(self, logger=None):
         """
@@ -655,6 +654,7 @@ class Environment(BaseEnv):
         res["opponent_action_class"] = self.opponent_action_class
         res["opponent_class"] = self.opponent_class
         res["opponent_init_budget"] = self.opponent_init_budget
+        res["name"] = self.name
         return res
 
     def get_params_for_runner(self):
@@ -704,5 +704,6 @@ class Environment(BaseEnv):
         res["opponent_class"] = self.opponent_class
         res["opponent_init_budget"] = self.opponent_init_budget
         res["grid_layout"] = self.grid_layout
+        res["name_env"] = self.name
         # TODO make a test for that
         return res
