@@ -56,6 +56,8 @@ class RemoteEnv(Process):
         self.space_prng.seed(seed=self.seed_used)
         self.backend = self.env_params["backendClass"]()
         del self.env_params["backendClass"]
+        chronics_handler = self.env_params["chronics_handler"]
+
         self.env = Environment(**self.env_params, backend=self.backend)
         self.env.chronics_handler.shuffle(shuffler=lambda x: x[self.space_prng.choice(len(x), size=len(x), replace=False)])
 
@@ -69,14 +71,18 @@ class RemoteEnv(Process):
         # TODO dirty hack because of wrong chronics
         # need to check!!!
         conv = False
-        obs = None
+        obs_v = None
         while not conv:
             try:
                 obs = self.env.reset()
-                conv = True
+                obs_v = obs.to_vect()
+                if np.all(np.isfinite(obs_v)):
+                    # i make sure that everything is not Nan
+                    # other i consider it's "divergence" so "game over"
+                    conv = True
             except:
                 pass
-        return obs
+        return obs_v
 
     def run(self):
         if self.env is None:
@@ -88,17 +94,18 @@ class RemoteEnv(Process):
                 self.remote.send((self.env.observation_space, self.env.action_space))
             elif cmd == 's':
                 # perform a step
+                data = self.env.action_space.from_vect(data)
                 obs, reward, done, info = self.env.step(data)
-                if done:
+                obs_v = obs.to_vect()
+                if done or np.any(~np.isfinite(obs_v)):
                     # if done do a reset
-                    obs = self.get_obs_ifnotconv()
-                self._clean_observation(obs)
-                self.remote.send((obs, reward, done, info))
+                    obs_v = self.get_obs_ifnotconv()
+                self.remote.send((obs_v, reward, done, info))
             elif cmd == 'r':
                 # perfom a reset
-                obs = self.get_obs_ifnotconv()
-                self._clean_observation(obs)
-                self.remote.send(obs)
+                obs_v = self.get_obs_ifnotconv()
+                # self._clean_observation(obs)
+                self.remote.send(obs_v)
             elif cmd == 'c':
                 # close everything
                 self.env.close()
@@ -188,7 +195,6 @@ class MultiEnvironment(GridObjects):
     """
     def __init__(self, nb_env, env):
         GridObjects.__init__(self)
-        # self.init_grid(env)
         self.imported_env = env
         self.nb_env = nb_env
 
@@ -214,13 +220,14 @@ class MultiEnvironment(GridObjects):
 
     def _send_act(self, actions):
         for remote, action in zip(self._remotes, actions):
-            remote.send(('s', action))
+            remote.send(('s', action.to_vect()))
         self._waiting = True
 
     def _wait_for_obs(self):
         results = [remote.recv() for remote in self._remotes]
         self._waiting = False
         obs, rews, dones, infos = zip(*results)
+        obs = [self.imported_env.observation_space.from_vect(ob) for ob in obs]
         return np.stack(obs), np.stack(rews), np.stack(dones), infos
 
     def step(self, actions):
@@ -279,7 +286,7 @@ class MultiEnvironment(GridObjects):
         """
         for remote in self._remotes:
             remote.send(('r', None))
-        res = [remote.recv() for remote in self._remotes]
+        res = [self.imported_env.observation_space.from_vect(remote.recv()) for remote in self._remotes]
         return np.stack(res)
 
     def close(self):
