@@ -6,17 +6,17 @@
 # SPDX-License-Identifier: MPL-2.0
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 import os
-import sys
-import numpy as np
 import warnings
 import time
 import imageio
 
+import argparse
+
 from grid2op.Exceptions import Grid2OpException
 from grid2op.PlotGrid.PlotMatplot import PlotMatplot
-from grid2op.Exceptions.PlotExceptions import PyGameQuit
 from grid2op.Episode.EpisodeData import EpisodeData
 
+
 class EpisodeReplay(object):
     """
     This class allows to see visually what an agent has done during an episode. It uses for now the "PlotPygame" as the
@@ -48,7 +48,7 @@ class EpisodeReplay(object):
         to save tha log of the agent.
 
     episode_data: :class:`grid2op.EpisodeData.EpisodeData`, optional
-        The last data of the episode inspected.
+        The last data of the episode inspected.replay_cli
     """
     def __init__(self, agent_path):
         if not os.path.exists(agent_path):
@@ -102,7 +102,7 @@ class EpisodeReplay(object):
             fig = plot_runner.plot_obs(observation=obs, figure=figure, redraw=True)
             if figure is None and display:
                 fig.show()
-            else:
+            elif display:
                 fig.canvas.draw()
 
             # Store figure for re-use
@@ -127,15 +127,15 @@ class EpisodeReplay(object):
             # Try to compress
             try:
                 from pygifsicle import optimize
-                optimize(gif_path)
+                optimize(gif_path, options=["-w", "--no-conserve-memory"])
             except:
                 warn_msg = "Failed to optimize .GIF size, but gif is still saved:\n" \
                            "Install dependencies to reduce size by ~3 folds\n" \
                            "apt-get install gifsicle && pip3 install pygifsicle"
                 warnings.warn(warn_msg)
 
-def replay_cli():    
-    import argparse
+
+def episode_replay_cli():
     parser = argparse.ArgumentParser(description="EpisodeReplay")
     parser.add_argument("--agent_path", required=True, type=str)
     parser.add_argument("--episode_id", required=True, type=str)
@@ -143,12 +143,20 @@ def replay_cli():
     parser.add_argument("--fps", required=False, default=2.0, type=float)
     parser.add_argument("--gif_name", required=False, default=None, type=str)
     args = parser.parse_args()
+    return args
+
+
+def main(args=None):
+    if args is None:
+        args = episode_replay_cli()
     er = EpisodeReplay(args.agent_path)
     er.replay_episode(args.episode_id,
                       fps=args.fps,
                       gif_name=args.gif_name,
                       display=args.display)
 
+
 # Dev / Test by running this file
 if __name__ == "__main__":
-    replay_cli()
+    args = episode_replay_cli()
+    main(args)
