@@ -54,7 +54,8 @@ class SerializableSpace(GridObjects, RandomObject):
         spaces they are made of.
 
     """
-    def __init__(self, gridobj,
+    def __init__(self,
+                 gridobj,
                  subtype=object):
         """
 
@@ -72,11 +73,11 @@ class SerializableSpace(GridObjects, RandomObject):
 
         GridObjects.__init__(self)
         RandomObject.__init__(self)
-
         self.init_grid(gridobj)
 
-        self.subtype = subtype
-        self._template_obj = self.subtype(gridobj=self)
+        self._init_subtype = subtype  # do not use, use to save restore only !!!
+        self.subtype = subtype.init_grid(gridobj)
+        self._template_obj = self.subtype()
         self.n = self._template_obj.size()
 
         self.global_vars = None
@@ -120,10 +121,9 @@ class SerializableSpace(GridObjects, RandomObject):
 
         gridobj = GridObjects.from_dict(dict_)
 
-        actionClass_str = extract_from_dict(dict_, "subtype", str)
+        actionClass_str = extract_from_dict(dict_, "_init_subtype", str)
         actionClass_li = actionClass_str.split('.')
 
-        # pdb.set_trace()
         if actionClass_li[-1] in globals():
             subtype = globals()[actionClass_li[-1]]
         else:
@@ -178,8 +178,8 @@ class SerializableSpace(GridObjects, RandomObject):
         """
         res = super().to_dict()
 
-        save_to_dict(res, self, "subtype", lambda x: re.sub("(<class ')|('>)", "", "{}".format(x)))
-
+        save_to_dict(res, self, "_init_subtype",
+                     lambda x: re.sub("(<class ')|(\\.init_grid\\.<locals>\\.res)|('>)", "", "{}".format(x)))
         return res
 
     def size(self):
