@@ -10,16 +10,24 @@ import pdb
 import warnings
 from grid2op.tests.helper_path_test import *
 from grid2op.Environment import MultiEnvironment
-from grid2op.MakeEnv import make_new
+from grid2op.MakeEnv import make
+from grid2op.Observation import CompleteObservation
 
 
 class TestLoadingMultiEnv(unittest.TestCase):
     def test_creation_multienv(self):
-        nb_env = 1
+        nb_env = 2
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            with make_new("rte_case5_example", test=True) as env:
+            with make("rte_case5_example", test=True) as env:
                 multi_envs = MultiEnvironment(env=env, nb_env=nb_env)
+        obss, rewards, dones, infos = multi_envs.step([env.action_space() for _ in range(nb_env)])
+        for ob in obss:
+            assert isinstance(ob, CompleteObservation)
+
+        obss = multi_envs.reset()
+        for ob in obss:
+            assert isinstance(ob, CompleteObservation)
         multi_envs.close()
 
 
