@@ -17,7 +17,7 @@ from grid2op.Environment import Environment
 from grid2op.Backend import PandaPowerBackend
 from grid2op.Parameters import Parameters
 from grid2op.Chronics import ChronicsHandler, GridStateFromFile, ChangeNothing
-from grid2op.MakeEnv import make_new
+from grid2op.MakeEnv import make
 from grid2op.Action import BaseAction
 
 
@@ -67,7 +67,8 @@ class TestRedispatch(HelperTests):
                                chronics_handler=self.chronics_handler,
                                parameters=self.env_params,
                                names_chronics_to_backend=self.names_chronics_to_backend,
-                               actionClass=BaseAction)
+                               actionClass=BaseAction,
+                               name="test_redisp_env1")
         self.array_double_dispatch = np.array([0.,  10.,  20.,   0., -30.])
 
     def tearDown(self):
@@ -298,14 +299,14 @@ class TestRedispatchChangeNothingEnvironment(HelperTests):
                                chronics_handler=self.chronics_handler,
                                parameters=self.env_params,
                                names_chronics_to_backend=self.names_chronics_to_backend,
-                               actionClass=BaseAction)
+                               actionClass=BaseAction,
+                               name="test_redisp_env2")
 
     def tearDown(self):
         self.env.close()
 
     def test_redispatch_generator_off(self):
         """ Redispatch a turned off generator is illegal """
-
         # Step into simulation once
         nothing_act = self.env.action_space()
         obs, reward, done, info = self.env.step(nothing_act)
@@ -329,7 +330,7 @@ class TestRedispTooLowHigh(HelperTests):
     def setUp(self) -> None:
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            self.env = make_new("rte_case14_redisp", test=True)
+            self.env = make("rte_case14_redisp", test=True)
 
         # i don't want to be bother by ramps in these test (note that is NOT recommended to change that)
         self.env.gen_max_ramp_down[:] = 5000
@@ -397,7 +398,7 @@ class TestLoadingBackendPandaPower(HelperTests):
         # powergrid
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            self.env = make_new("rte_case14_test", test=True)
+            self.env = make("rte_case14_test", test=True)
 
     def tearDown(self):
         self.env.close()
@@ -535,6 +536,7 @@ class TestLoadingBackendPandaPower(HelperTests):
             assert np.all(obs.prod_p[0:2] >= -obs.gen_pmin[0:2])
             assert np.abs(np.sum(obs.actual_dispatch)) <= self.tol_one
             assert len(info['exception']) == 0, "error at iteration {}".format(i)
+
             i += 1
             obs_init = obs
             if i >= max_iter:
@@ -554,7 +556,7 @@ class TestLoadingAcceptAlmostZeroSumRedisp(HelperTests):
         # powergrid
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            self.env = make_new("rte_case14_test", test=True)
+            self.env = make("rte_case14_test", test=True)
 
     def tearDown(self):
         self.env.close()
